################################################################################
# The <ergm.etagrad> function caculates and returns the gradient of eta
# mapped from theta using the etamap object created by <ergm.etamap>. If the
# gradient is only intended to be a multiplier for some vector, the more
# efficient <ergm.etagradmult> is recommended.
#
# --PARAMETERS--
#   theta :  the vector of curved model parameters 
#   etamap:  the list constituting the theta-> eta mapping that is returned by
#            <ergm.etamap>
#
# --RETURNED--
#   etagrad: a matrix of the gradient of eta 
#
################################################################################

ergm.etagrad <- function(theta, etamap) {
  etagrad <- matrix(0, length(theta), etamap$etalength)
  ec <- etamap$canonical
# Set gradient for canonical parameters to the identity matrix
  etagrad[ec>0, ec[ec>0]] <- diag(sum(ec>0))
  if(length(etamap$curved)>0) {
    for(i in 1:length(etamap$curved)) {
      cm <- etamap$curved[[i]]
      etagrad[cm$from,cm$to] <- cm$gradient(theta[cm$from], length(cm$to), cm$cov)  #Added by CTB on 1/28/06
    }
  }
  etagrad[is.infinite(etagrad)] <- 10000*sign(etagrad)[is.infinite(etagrad)]
  etagrad
}















































































